@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2020 Synchrotron SOLEIL
+ * Copyright (C) 2003-2020, 2022, 2023 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -43,7 +43,7 @@ static inline HklParameter *hkl_axis_copy_real(const HklParameter *base)
 	HklAxis *self = container_of(base, HklAxis, parameter);
 	HklAxis *dup;
 
-	dup = HKL_MALLOC(HklAxis);
+	dup = g_new(HklAxis, 1);
 
 	*dup = *self;
 
@@ -85,10 +85,10 @@ static inline int hkl_axis_set_value_real(HklParameter *self, double value,
 	hkl_error (error == NULL || *error == NULL);
 
 	if(!hkl_parameter_value_set_real(self, value, unit_type, error)){
-		g_assert (error == NULL || *error != NULL);
+		hkl_assert (error == NULL || *error != NULL);
 		return FALSE;
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
 	hkl_axis_update(axis);
 
@@ -285,15 +285,16 @@ HklParameter *hkl_parameter_new_rotation(const char *name, HklVector const *axis
 {
 	HklAxis axis0 = {
 		.parameter = { HKL_PARAMETER_DEFAULTS_ANGLE,
-			       .name = name,
-			       .punit = punit,
-			       .ops = &hkl_parameter_operations_axis,
+			.name = name,
+			.punit = punit,
+			.ops = &hkl_parameter_operations_axis,
+                        .type = Rotation(*axis_v),
 		},
 		.axis_v = *axis_v,
 		.q = {{1., 0., 0., 0.}},
 	};
 
-	HklAxis *self =  HKL_MALLOC(HklAxis);
+	HklAxis *self =  g_new(HklAxis, 1);
 
 	*self = axis0;
 
@@ -310,7 +311,7 @@ static inline HklParameter *hkl_rotation_with_origin_copy_real(const HklParamete
 						   HklRotationWithOrigin, axis);
 	HklRotationWithOrigin *dup;
 
-	dup = HKL_MALLOC(HklRotationWithOrigin);
+	dup = g_new(HklRotationWithOrigin, 1);
 
 	*dup = *self;
 
@@ -386,9 +387,10 @@ HklParameter *hkl_parameter_new_rotation_with_origin(const char *name,
 	HklRotationWithOrigin rotation0 = {
 		.axis = {
 			.parameter = { HKL_PARAMETER_DEFAULTS_ANGLE,
-				       .name = name,
-				       .punit = punit,
-				       .ops = &hkl_parameter_operations_rotation_with_origin,
+				.name = name,
+				.punit = punit,
+				.ops = &hkl_parameter_operations_rotation_with_origin,
+                                .type = RotationWithOrigin(*axis_v, *origin),
 			},
 			.axis_v = *axis_v,
 			.q = {{1., 0., 0., 0.}},
@@ -396,7 +398,7 @@ HklParameter *hkl_parameter_new_rotation_with_origin(const char *name,
 		.origin = *origin,
 	};
 
-	HklRotationWithOrigin *self =  HKL_MALLOC(HklRotationWithOrigin);
+	HklRotationWithOrigin *self =  g_new(HklRotationWithOrigin, 1);
 
 	*self = rotation0;
 
@@ -412,7 +414,7 @@ static inline HklParameter *hkl_translation_copy_real(const HklParameter *base)
 	HklTranslation *self = container_of(base, HklTranslation, parameter);
 	HklTranslation *dup;
 
-	dup = HKL_MALLOC(HklTranslation);
+	dup = g_new(HklTranslation, 1);
 
 	*dup = *self;
 
@@ -494,14 +496,15 @@ HklParameter *hkl_parameter_new_translation(const char *name, HklVector const *a
 {
 	HklTranslation translation0 = {
 		.parameter = { HKL_PARAMETER_DEFAULTS_LENGTH,
-			       .name = name,
-			       .punit = punit,
-			       .ops = &hkl_parameter_operations_translation,
+			.name = name,
+			.punit = punit,
+			.ops = &hkl_parameter_operations_translation,
+                        .type = Translation(*axis_v),
 		},
 		.axis_v = *axis_v,
 	};
 
-	HklTranslation *self =  HKL_MALLOC(HklTranslation);
+	HklTranslation *self =  g_new(HklTranslation, 1);
 
 	*self = translation0;
 
