@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019 Synchrotron SOLEIL
+ * Copyright (C) 2003-2020, 2022, 2023 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -35,6 +35,7 @@
 #include "hkl-geometry-private.h"       // for _HklGeometry, etc
 #include "hkl-interval-private.h"       // for HklInterval
 #include "hkl-macros-private.h"         // for HKL_MALLOC
+#include "hkl-matrix-private.h"
 #include "hkl-parameter-private.h"      // for _HklParameter, etc
 #include "hkl-quaternion-private.h"     // for _HklQuaternion, etc
 #include "hkl-source-private.h"         // for HklSource, hkl_source_init
@@ -78,6 +79,21 @@ static size_t hkl_geometry_add_axis(HklGeometry *self, const HklParameter *axis)
 	return darray_size(self->axes) - 1;
 }
 
+static void hkl_vector_project_into_reciprocal_space(HklVector *self,
+                                                     const HklGeometry *geometry,
+                                                     const HklSample *sample)
+{
+        HklQuaternion qs = hkl_geometry_sample_rotation_get(geometry, sample);
+        HklMatrix RUB;
+        HklMatrix RUB_1;
+
+        hkl_quaternion_to_matrix(&qs, &RUB);
+        hkl_matrix_times_matrix(&RUB, hkl_sample_UB_get(sample));
+        hkl_matrix_inv(&RUB, &RUB_1);
+
+        hkl_matrix_times_vector(&RUB_1, self);
+}
+
 /*******************/
 /* HklHolderConfig */
 /*******************/
@@ -86,7 +102,7 @@ static struct HklHolderConfig *hkl_holder_config_new(void)
 {
 	struct HklHolderConfig *self;
 
-	self = HKL_MALLOC(struct HklHolderConfig);
+	self = g_new(struct HklHolderConfig, 1);
 
 	self->gc = 1;
 	self->idx = NULL;
@@ -124,7 +140,7 @@ static void hkl_holder_config_unref(struct HklHolderConfig *self)
 static HklHolder *hkl_holder_new(HklGeometry *geometry)
 {
 	static HklQuaternion q0 = {{1, 0, 0, 0}};
-	HklHolder *self = HKL_MALLOC(HklHolder);
+	HklHolder *self = g_new(HklHolder, 1);
 
 	self->config = hkl_holder_config_new();
 	self->geometry = geometry;
@@ -135,7 +151,7 @@ static HklHolder *hkl_holder_new(HklGeometry *geometry)
 
 static HklHolder *hkl_holder_new_copy(HklHolder *src, HklGeometry *geometry)
 {
-	HklHolder *self = HKL_MALLOC(HklHolder);
+	HklHolder *self = g_new(HklHolder, 1);
 
 	self->config = hkl_holder_config_ref(src->config);
 	self->geometry = geometry;
@@ -174,7 +190,7 @@ static void hkl_holder_update(HklHolder *self)
 		p = darray_item(self->geometry->axes, self->config->idx[i]);
 		q = hkl_parameter_quaternion_get(p);
 		if(NULL == q)
-			break;
+			continue;
 		else
 			hkl_quaternion_times_quaternion(&self->q, q);
 	}
@@ -197,6 +213,21 @@ static HklParameter * hkl_holder_add_axis_if_not_present(const HklHolder *self,
 	return res;
 }
 
+HklParameter *hkl_holder_add_parameter(HklHolder *self,
+                                       const char *name,
+                                       const HklUnit *punit)
+{
+        HklParameter parameter = {
+                HKL_PARAMETER_DEFAULTS_LENGTH,
+                .name = name,
+                .punit = punit
+        };
+
+	int idx = hkl_geometry_add_axis(self->geometry, &parameter);
+
+	return hkl_holder_add_axis_if_not_present(self, idx);
+}
+
 HklParameter *hkl_holder_add_rotation(HklHolder *self,
 				      const char *name,
 				      double x, double y, double z,
@@ -272,12 +303,20 @@ const HklGeometryOperations hkl_geometry_operations_defaults = { HKL_GEOMETRY_OP
 HklGeometry *hkl_geometry_new(const HklFactory *factory,
 			      const HklGeometryOperations *ops)
 {
-	HklGeometry *g = NULL;
+	HklGeometry *g = g_new(HklGeometry, 1);
 
-	g = HKL_MALLOC(HklGeometry);
-	g->ops = ops;
+        if(NULL == factory){
+                static const HklFactory readonly = {
+                        .name = "readonly",
+                        .description = "Readonly geometry which can be customise by the user",
+                };
+                g->factory = &readonly;
+                g->ops = &hkl_geometry_operations_defaults;
+        } else {
+                g->factory = factory;
+                g->ops = ops;
+        }
 
-	g->factory = factory;
 	hkl_source_init(&g->source, 1.54, 1, 0, 0);
 	darray_init(g->axes);
 	darray_init(g->holders);
@@ -295,12 +334,10 @@ HklGeometry *hkl_geometry_new(const HklFactory *factory,
  **/
 HklGeometry *hkl_geometry_new_copy(const HklGeometry *src)
 {
-	HklGeometry *self = NULL;
+	HklGeometry *self = g_new(HklGeometry, 1);
 	HklParameter **axis;
 	HklHolder **holder;
 
-	self = HKL_MALLOC(HklGeometry);
-
 	*self = *src;
 
 	/* copy the axes */
@@ -576,9 +613,6 @@ int hkl_geometry_get_axis_idx_by_name(const HklGeometry *self, const char *name)
 	uint i = 0;
 	HklParameter **axis;
 
-	if (!self || !name)
-		return -1;
-
 	darray_foreach(axis, self->axes){
 		if (!strcmp((*axis)->name, name))
 			return i;
@@ -652,15 +686,15 @@ int hkl_geometry_axis_values_set(HklGeometry *self,
 	HklParameter **axis;
 
 	hkl_error (error == NULL || *error == NULL);
-	g_assert(n_values == darray_size(self->axes));
+	hkl_assert(n_values == darray_size(self->axes));
 
 	darray_foreach(axis, self->axes){
 		if(!hkl_parameter_value_set(*axis, values[i++], unit_type, error)){
-			g_assert (error == NULL || *error != NULL);
+			hkl_assert (error == NULL || *error != NULL);
 			return FALSE;
 		}
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
 	hkl_geometry_update(self);
 
@@ -706,13 +740,13 @@ int hkl_geometry_set_values_v(HklGeometry *self, HklUnitEnum unit_type, GError *
 		if(!hkl_parameter_value_set(*axis,
 					    va_arg(ap, double),
 					    unit_type, error)){
-			g_assert (error == NULL || *error != NULL);
+			hkl_assert (error == NULL || *error != NULL);
 			va_end(ap);
 			hkl_geometry_update(self);
 			return FALSE;
 		}
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
 	va_end(ap);
 
@@ -737,9 +771,6 @@ double hkl_geometry_distance(const HklGeometry *self,
 	double value1, value2;
 	double distance = 0.;
 
-	if (!self || !ref)
-		return 0.;
-
 	for(i=0; i<darray_size(self->axes); ++i){
 		value1 = darray_item(self->axes, i)->_value;
 		value2 = darray_item(ref->axes, i)->_value;
@@ -760,24 +791,13 @@ double hkl_geometry_distance_orthodromic(const HklGeometry *self,
 					 const HklGeometry *ref)
 {
 	size_t i;
-	double value1, value2;
 	double distance = 0.;
 
-	if (!self || !ref)
-		return 0.;
-
 	for(i=0; i<darray_size(self->axes); ++i){
-		double d;
-
-		value1 = darray_item(self->axes, i)->_value;
-		value2 = darray_item(ref->axes, i)->_value;
-		d = fabs(gsl_sf_angle_restrict_symm(value2) - gsl_sf_angle_restrict_symm(value1));
-		/* as M_PI and -M_PI are included in the GSL restriction */
-		if (d > M_PI)
-			d = 2*M_PI - d;
-		distance += d;
+                distance += hkl_parameter_orthodromic_distance_get(
+                        darray_item(self->axes, i),
+                        darray_item(ref->axes, i)->_value);
 	}
-
 	return distance;
 }
 
@@ -850,9 +870,9 @@ int hkl_geometry_closest_from_geometry_with_range(HklGeometry *self,
 	}
 	if(!ko){
 		for(i=0;i<len;++i)
-			hkl_parameter_value_set(darray_item(self->axes, i),
-						values[i],
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(darray_item(self->axes, i),
+                                                       values[i],
+                                                       HKL_UNIT_DEFAULT, NULL));
 		hkl_geometry_update(self);
 	}
 	return ko;
@@ -876,13 +896,13 @@ HklQuaternion hkl_geometry_sample_rotation_get(const HklGeometry *self,
 
 /**
  * hkl_geometry_detector_rotation_get:
- * @self: the self @HklGeometry@
- * @detector: the @HklDetector@
+ * @self: the self #HklGeometry
+ * @detector: the #HklDetector
  *
  * return the rotation part of the given detector in the laboratory
  * basis.
  *
- * Returns: the rotation express as a quaternion.
+ * Returns: the rotation express as a #HklQuaternion.
  **/
 HklQuaternion hkl_geometry_detector_rotation_get(const HklGeometry *self,
 						 const HklDetector *detector)
@@ -921,17 +941,78 @@ HklHolder *hkl_geometry_detector_holder_get(const HklGeometry *self,
 	return self->ops->detector_holder_get(self, detector);
 }
 
+/**
+ * hkl_geometry_ki_get:
+ * @self: the self #HklGeometry
+ *
+ * return the ki vector in the laboratory
+ * basis.
+ *
+ * Returns: the ki vector as #HklVector.
+ **/
 HklVector hkl_geometry_ki_get(const HklGeometry *self)
 {
 	return self->ops->ki_get(self);
 }
 
+/**
+ * hkl_geometry_ki_abc_get:
+ * @self: the self #HklGeometry
+ * @sample: the sample #HklSample
+ *
+ * return the ki vector in the sample lattice reciprocal coordinates
+ * basis.
+ *
+ * Returns: the ki vector as an #HklVector.
+ **/
+HklVector hkl_geometry_ki_abc_get(const HklGeometry *self,
+                                  const HklSample *sample)
+{
+        HklVector ki_abc = hkl_geometry_ki_get(self);
+
+        hkl_vector_project_into_reciprocal_space(&ki_abc, self, sample);
+
+        return ki_abc;
+}
+
+/**
+ * hkl_geometry_kf_get:
+ * @self: the self #HklGeometry
+ * @detector: the #HklDetector
+ *
+ * return the kf vector in the laboratory
+ * basis.
+ *
+ * Returns: the kf vector.
+ **/
 HklVector hkl_geometry_kf_get(const HklGeometry *self,
 			      const HklDetector *detector)
 {
 	return self->ops->kf_get(self, detector);
 }
 
+/**
+ * hkl_geometry_kf_abc_get:
+ * @self: the self #HklGeometry
+ * @detector: the #HklDetector
+ * @sample: the sample #HklSample
+ *
+ * return the kf vector in the sample reciprocal lattice coordinates
+ * basis.
+ *
+ * Returns: the kf vector.
+ **/
+HklVector hkl_geometry_kf_abc_get(const HklGeometry *self,
+                                  const HklDetector *detector,
+                                  const HklSample* sample)
+{
+        HklVector kf_abc = hkl_geometry_kf_get(self, detector);
+
+        hkl_vector_project_into_reciprocal_space(&kf_abc, self, sample);
+
+        return kf_abc;
+}
+
 /*******************/
 /* HklGeometryList */
 /*******************/
@@ -945,9 +1026,7 @@ HklVector hkl_geometry_kf_get(const HklGeometry *self,
  **/
 HklGeometryList *hkl_geometry_list_new(void)
 {
-	HklGeometryList *self;
-
-	self = HKL_MALLOC(HklGeometryList);
+	HklGeometryList *self = g_new(HklGeometryList, 1);
 
 	list_head_init(&self->items);
 	self->n_items = 0;
@@ -969,10 +1048,7 @@ HklGeometryList *hkl_geometry_list_new_copy(const HklGeometryList *self)
 	HklGeometryList *dup;
 	HklGeometryListItem *item;
 
-	if (!self)
-		return NULL;
-
-	dup = HKL_MALLOC(HklGeometryList);
+	dup = g_new(HklGeometryList, 1);
 
 	list_head_init(&dup->items);
 	/* now copy the item arrays */
@@ -1021,9 +1097,11 @@ void hkl_geometry_list_add(HklGeometryList *self, const HklGeometry *geometry)
 			return;
 	}
 
-	list_add_tail(&self->items,
-		      &hkl_geometry_list_item_new(geometry)->list);
-	self->n_items += 1;
+        item = hkl_geometry_list_item_new(geometry);
+        if (NULL != item){
+                list_add_tail(&self->items, &item->list);
+                self->n_items += 1;
+        }
 }
 
 /**
@@ -1143,14 +1221,15 @@ void hkl_geometry_list_fprintf(FILE *f, const HklGeometryList *self)
 	uint i = 0;
 	double value;
 
-	if(!self)
-		return;
-
 	fprintf(f, "multiply method: %p \n", self->multiply);
 	if(self->n_items){
 		HklGeometryListItem *item;
 		HklParameter **axis;
 
+                item = list_top(&self->items, HklGeometryListItem, list);
+                if(!item)
+                        return;
+
 		fprintf(f, "    ");
 		darray_foreach(axis, list_top(&self->items, HklGeometryListItem, list)->geometry->axes){
 			fprintf(f, "%19s", (*axis)->name);
@@ -1186,7 +1265,7 @@ void hkl_geometry_list_multiply(HklGeometryList *self)
 	uint len = self->n_items;
 	HklGeometryListItem *item;
 
-	if(!self || !self->multiply)
+	if(!self->multiply)
 		return;
 
 	/*
@@ -1200,7 +1279,7 @@ void hkl_geometry_list_multiply(HklGeometryList *self)
 }
 
 static void perm_r(HklGeometryList *self, const HklGeometry *ref,
-		   const HklGeometry *geometry, const int perm[],
+		   const HklGeometry *geometry,
 		   const unsigned int axis_idx)
 {
 	if (axis_idx == darray_size(geometry->axes)){
@@ -1210,8 +1289,9 @@ static void perm_r(HklGeometryList *self, const HklGeometry *ref,
 			self->n_items++;
 		}
 	}else{
-		if(perm[axis_idx]){
-			HklParameter *axis = darray_item(geometry->axes, axis_idx);
+                HklParameter *axis = darray_item(geometry->axes, axis_idx);
+
+		if(hkl_parameter_is_permutable(axis)){
 			const double max = axis->range.max;;
 			const double value0 = axis->_value;
 			double value;
@@ -1219,7 +1299,7 @@ static void perm_r(HklGeometryList *self, const HklGeometry *ref,
 			value = value0;
 			do{
 				/* fprintf(stdout, "\n%d %s, %f", axis_idx, hkl_axis_get_name(axis), value * HKL_RADTODEG); */
-				perm_r(self, ref, geometry, perm, axis_idx + 1);
+				perm_r(self, ref, geometry, axis_idx + 1);
 				value +=  2*M_PI;
 				if(value <= (max + HKL_EPSILON)){
 					/* optimisation here: */
@@ -1236,49 +1316,38 @@ static void perm_r(HklGeometryList *self, const HklGeometry *ref,
 			/* restore the initial value */
 			axis->_value = value0;
 		} else
-			perm_r(self, ref, geometry, perm, axis_idx + 1);
+			perm_r(self, ref, geometry, axis_idx + 1);
 	}
 }
 
 void hkl_geometry_list_multiply_from_range(HklGeometryList *self)
 {
-	uint i;
-	uint len = self->n_items;
-	const HklGeometryListItem *item;
-
-	if(!self)
-		return;
-
+        size_t i;
+        size_t len;
+        const HklGeometryListItem *item;
 	/*
 	 * warning this method change the self->len so we need to save it
 	 * before using the recursive perm_r calls
 	 */
+        len =  self->n_items;
 
 	for(i=0, item=list_top(&self->items, HklGeometryListItem, list);
-	    i<len;
+	    i<len && NULL != item;
 	    ++i, item=list_next(&self->items, item, list)){
-		HklGeometry *geometry;
-		HklParameter **axis;
-		int *perm;
-		size_t j = 0;
-
-		geometry = hkl_geometry_new_copy(item->geometry);
-		perm = alloca(darray_size(geometry->axes) * sizeof(*perm));
+                HklParameter **axis;
+		HklGeometry *geometry = hkl_geometry_new_copy(item->geometry);
 
 		/* find axes to permute and the first solution of thoses axes */
-		darray_foreach(axis, geometry->axes){
-			perm[j] = hkl_parameter_is_valid(*axis);
-			/* fprintf(stdout, "%d %d\n", j, perm[j]); */
-			if (perm[j])
+                darray_foreach(axis, geometry->axes){
+			if (hkl_parameter_is_permutable(*axis))
 				hkl_parameter_value_set_smallest_in_range(*axis);
-			++j;
 		}
 		/*
 		 * fprintf(stdout, "FIRST SOLUTION\n");
 		 * hkl_geometry_fprintf(stdout, geometry);
 		 */
 
-		perm_r(self, item->geometry, geometry, perm, 0);
+		perm_r(self, item->geometry, geometry, 0);
 		hkl_geometry_free(geometry);
 	}
 }
@@ -1315,12 +1384,7 @@ void hkl_geometry_list_remove_invalid(HklGeometryList *self)
  **/
 HklGeometryListItem *hkl_geometry_list_item_new(const HklGeometry *geometry)
 {
-	HklGeometryListItem *self;
-
-	if(!geometry)
-		return NULL;
-
-	self = HKL_MALLOC(HklGeometryListItem);
+	HklGeometryListItem *self = g_new(HklGeometryListItem, 1);
 
 	self->geometry = hkl_geometry_new_copy(geometry);
 
@@ -1337,12 +1401,7 @@ HklGeometryListItem *hkl_geometry_list_item_new(const HklGeometry *geometry)
  **/
 HklGeometryListItem *hkl_geometry_list_item_new_copy(const HklGeometryListItem *self)
 {
-	HklGeometryListItem *dup;
-
-	if(!self)
-		return NULL;
-
-	dup = HKL_MALLOC(HklGeometryListItem);
+	HklGeometryListItem *dup = g_new(HklGeometryListItem, 1);
 
 	dup->geometry = hkl_geometry_new_copy(self->geometry);
 
@@ -1357,9 +1416,6 @@ HklGeometryListItem *hkl_geometry_list_item_new_copy(const HklGeometryListItem *
  **/
 void hkl_geometry_list_item_free(HklGeometryListItem *self)
 {
-	if(!self)
-		return;
-
 	hkl_geometry_free(self->geometry);
 	free(self);
 }
