@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2016 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2022, 2023, 2024 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -84,63 +84,140 @@ static int check_lattice_param(double a, double b, double c,
  **/
 HklLattice *hkl_lattice_new(double a, double b, double c,
 			    double alpha, double beta, double gamma,
-			    GError **error)
+			    GError **err)
 {
 	HklLattice *self = NULL;
+	HklParameter *parameter = NULL;
 	double volume;
 
-	hkl_error (error == NULL || *error == NULL);
+	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
-	if(!check_lattice_param(a, b, c, alpha, beta, gamma, &volume, error))
+	if(!check_lattice_param(a, b, c, alpha, beta, gamma, &volume, err))
 	{
-		g_assert (error == NULL || *error != NULL);
+		hkl_assert (err == NULL || *err != NULL);
 		return FALSE;
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (err == NULL || *err == NULL);
 
-	self = HKL_MALLOC(HklLattice);
+	self = g_new(HklLattice, 1);
 
-	self->a = hkl_parameter_new("a", "The length of the first lattice vector",
-				    0, a, a+10,
-				    TRUE, TRUE,
-				    &hkl_unit_length_nm,
-				    &hkl_unit_length_nm);
-	self->b = hkl_parameter_new("b", "The length of the second lattice vector",
-				    0, b, b+10,
-				    TRUE, TRUE,
-				    &hkl_unit_length_nm,
-				    &hkl_unit_length_nm);
-	self->c = hkl_parameter_new("c", "The length of the third lattice vector",
-				    0, c, c+10,
-				    TRUE, TRUE,
-				    &hkl_unit_length_nm,
-				    &hkl_unit_length_nm);
-	self->alpha = hkl_parameter_new("alpha",
-					"The angle between the second and third lattice vector",
-					-M_PI, alpha, M_PI,
-					TRUE, TRUE,
-					&hkl_unit_angle_rad,
-					&hkl_unit_angle_deg);
-	self->beta = hkl_parameter_new("beta",
-				       "The angle between the first and third lattice vector",
-				       -M_PI, beta, M_PI,
-				       TRUE, TRUE,
-				       &hkl_unit_angle_rad,
-				       &hkl_unit_angle_deg);
-	self->gamma = hkl_parameter_new("gamma",
-					"The angle between the first and second lattice vector",
-					-M_PI, gamma, M_PI,
-					TRUE, TRUE,
-					&hkl_unit_angle_rad,
-					&hkl_unit_angle_deg);
-	self->volume = hkl_parameter_new("volume",
-					 "The volume of the lattice",
-					 0, volume, a*b*c,
-					 FALSE, FALSE,
-					 &hkl_unit_length_nm,
-					 &hkl_unit_length_nm);
+	/* a */
+	parameter = hkl_parameter_new("a", "The length of the first lattice vector",
+				      0, a, a+10,
+				      TRUE, TRUE,
+				      &hkl_unit_length_nm,
+				      &hkl_unit_length_nm, err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto failed;
+	}else{
+		self->a = parameter;
+	}
+
+	/* b */
+	parameter = hkl_parameter_new("b", "The length of the second lattice vector",
+				      0, b, b+10,
+				      TRUE, TRUE,
+				      &hkl_unit_length_nm,
+				      &hkl_unit_length_nm,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_a;
+	}else{
+		self->b = parameter;
+	}
+
+	/* c */
+	parameter = hkl_parameter_new("c", "The length of the third lattice vector",
+				      0, c, c+10,
+				      TRUE, TRUE,
+				      &hkl_unit_length_nm,
+				      &hkl_unit_length_nm,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_b;
+	}else{
+		self->c = parameter;
+	}
+
+	/* alpha */
+	parameter = hkl_parameter_new("alpha",
+				      "The angle between the second and third lattice vector",
+				      -M_PI, alpha, M_PI,
+				      TRUE, TRUE,
+				      &hkl_unit_angle_rad,
+				      &hkl_unit_angle_deg,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_c;
+	}else{
+		self->alpha = parameter;
+	}
+
+	/* beta */
+	parameter = hkl_parameter_new("beta",
+				      "The angle between the first and third lattice vector",
+				      -M_PI, beta, M_PI,
+				      TRUE, TRUE,
+				      &hkl_unit_angle_rad,
+				      &hkl_unit_angle_deg,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_alpha;
+	}else{
+		self->beta = parameter;
+	}
+
+	/* gamma */
+	parameter = hkl_parameter_new("gamma",
+				      "The angle between the first and second lattice vector",
+				      -M_PI, gamma, M_PI,
+				      TRUE, TRUE,
+				      &hkl_unit_angle_rad,
+				      &hkl_unit_angle_deg,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_beta;
+	}else{
+		self->gamma = parameter;
+	}
+
+	/* volume */
+	parameter = hkl_parameter_new("volume",
+				      "The volume of the lattice",
+				      0, volume, a*b*c,
+				      FALSE, FALSE,
+				      &hkl_unit_length_nm,
+				      &hkl_unit_length_nm,
+				      err);
+	if (!parameter) {
+		hkl_assert (err == NULL || *err != NULL);
+		goto free_gamma;
+	}else{
+		self->volume = parameter;
+	}
 
 	return self;
+
+free_gamma:
+	hkl_parameter_free(self->gamma);
+free_beta:
+	hkl_parameter_free(self->beta);
+free_alpha:
+	hkl_parameter_free(self->alpha);
+free_c:
+	hkl_parameter_free(self->c);
+free_b:
+	hkl_parameter_free(self->b);
+free_a:
+	hkl_parameter_free(self->a);
+failed:
+	return FALSE;
 }
 
 /**
@@ -153,9 +230,7 @@ HklLattice *hkl_lattice_new(double a, double b, double c,
  **/
 HklLattice *hkl_lattice_new_copy(const HklLattice *self)
 {
-	HklLattice *copy = NULL;
-
-	copy = HKL_MALLOC(HklLattice);
+	HklLattice *copy = g_new(HklLattice, 1);
 
 	copy->a = hkl_parameter_new_copy(self->a);
 	copy->b = hkl_parameter_new_copy(self->b);
@@ -208,10 +283,10 @@ void hkl_lattice_free(HklLattice *self)
 		_p = hkl_parameter_value_get((_parameter), HKL_UNIT_DEFAULT); \
 									\
 		if(!hkl_lattice_set(self, a, b, c, alpha, beta, gamma, HKL_UNIT_DEFAULT, (_error))){ \
-			g_assert ((_error) == NULL || *(_error) != NULL); \
+			hkl_assert ((_error) == NULL || *(_error) != NULL); \
 			return FALSE;					\
 		}							\
-		g_assert ((_error) == NULL || *(_error) == NULL);	\
+		hkl_assert ((_error) == NULL || *(_error) == NULL);	\
 		return hkl_parameter_init_copy(self->_p, (_parameter), (_error)); \
 	}while(0)
 
@@ -402,33 +477,93 @@ int hkl_lattice_set(HklLattice *self,
 {
 	hkl_error (error == NULL || *error == NULL);
 
-	double _a, _b, _c, _alpha, _beta, _gamma;
-	double _volume;
+	double previous_a = hkl_parameter_value_get(self->a, HKL_UNIT_DEFAULT);
+	double previous_b = hkl_parameter_value_get(self->b, HKL_UNIT_DEFAULT);
+	double previous_c = hkl_parameter_value_get(self->c, HKL_UNIT_DEFAULT);
+	double previous_alpha = hkl_parameter_value_get(self->alpha, HKL_UNIT_DEFAULT);
+	double previous_beta = hkl_parameter_value_get(self->beta, HKL_UNIT_DEFAULT);
+	double previous_gamma = hkl_parameter_value_get(self->gamma, HKL_UNIT_DEFAULT);
 
-	_a = convert_to_default(self->a, a, unit_type);
-	_b = convert_to_default(self->b, b, unit_type);
-	_c = convert_to_default(self->c, c, unit_type);
-	_alpha = convert_to_default(self->alpha, alpha, unit_type);
-	_beta = convert_to_default(self->beta, beta, unit_type);
-	_gamma = convert_to_default(self->gamma, gamma, unit_type);
+	double new_a = convert_to_default(self->a, a, unit_type);
+	double new_b = convert_to_default(self->b, b, unit_type);
+	double new_c = convert_to_default(self->c, c, unit_type);
+	double new_alpha = convert_to_default(self->alpha, alpha, unit_type);
+	double new_beta = convert_to_default(self->beta, beta, unit_type);
+	double new_gamma = convert_to_default(self->gamma, gamma, unit_type);
+	double new_volume;
 
 	/* need to do the conversion before the check */
-	if(!check_lattice_param(_a, _b, _c, _alpha, _beta, _gamma, &_volume, error)){
-		g_assert (error == NULL || *error != NULL);
+	if(!check_lattice_param(new_a, new_b, new_c, new_alpha, new_beta, new_gamma, &new_volume, error)){
+		hkl_assert (error == NULL || *error != NULL);
 		return FALSE;
 	}
-	g_assert (error == NULL || *error == NULL);
+	hkl_assert (error == NULL || *error == NULL);
 
-	hkl_parameter_value_set(self->a, _a, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->b, _b, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->c, _c, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->alpha, _alpha, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->beta, _beta, HKL_UNIT_DEFAULT, NULL);
-	hkl_parameter_value_set(self->gamma, _gamma, HKL_UNIT_DEFAULT, NULL);
+	/* check a */
+	if(!hkl_parameter_value_set(self->a, new_a, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto failed;
+	}
+	hkl_assert (error == NULL || *error == NULL);
 
-	hkl_parameter_value_set(self->volume, _volume, HKL_UNIT_DEFAULT, NULL);
+	if(!hkl_parameter_value_set(self->b, new_b, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_a;
+	}
+	hkl_assert (error == NULL || *error == NULL);
+
+	if(!hkl_parameter_value_set(self->c, new_c, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_b;
+	}
+	hkl_assert (error == NULL || *error == NULL);
+
+	if(!hkl_parameter_value_set(self->alpha, new_alpha, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_c;
+	}
+	hkl_assert (error == NULL || *error == NULL);
+
+	if(!hkl_parameter_value_set(self->beta, new_beta, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_alpha;
+	}
+	hkl_assert (error == NULL || *error == NULL);
+
+	if(!hkl_parameter_value_set(self->gamma, new_gamma, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_beta;
+	}
+	hkl_assert (error == NULL || *error == NULL);
+
+	if(!hkl_parameter_value_set(self->volume, new_volume, HKL_UNIT_DEFAULT, NULL)){
+		hkl_assert (error == NULL || *error != NULL);
+		goto restore_gamma;
+	}
+	hkl_assert (error == NULL || *error == NULL);
 
 	return TRUE;
+
+restore_gamma:
+	IGNORE(hkl_parameter_value_set(self->gamma, previous_gamma,
+				       HKL_UNIT_DEFAULT, NULL));
+restore_beta:
+	IGNORE(hkl_parameter_value_set(self->beta, previous_beta,
+				       HKL_UNIT_DEFAULT, NULL));
+restore_alpha:
+	IGNORE(hkl_parameter_value_set(self->alpha, previous_alpha,
+				       HKL_UNIT_DEFAULT, NULL));
+restore_c:
+	IGNORE(hkl_parameter_value_set(self->c, previous_c,
+				       HKL_UNIT_DEFAULT, NULL));
+restore_b:
+	IGNORE(hkl_parameter_value_set(self->b, previous_b,
+				       HKL_UNIT_DEFAULT, NULL));
+restore_a:
+	IGNORE(hkl_parameter_value_set(self->a, previous_a,
+				       HKL_UNIT_DEFAULT, NULL));
+failed:
+	return FALSE;
 }
 
 /**
@@ -442,7 +577,6 @@ int hkl_lattice_set(HklLattice *self,
  * @gamma: (out caller-allocates):
  *
  * get the lattice parameters
- * Return value: all the parameters
  **/
 void hkl_lattice_get(const HklLattice *self,
 		     double *a, double *b, double *c,
@@ -529,9 +663,6 @@ int hkl_lattice_get_1_B(const HklLattice *self, HklMatrix *B)
 	double e;
 	double f;
 
-	if(!self || !B)
-		return FALSE;
-
 	/*
 	 * first compute the B matrix
 	 * | a b c |
@@ -610,14 +741,14 @@ int hkl_lattice_reciprocal(const HklLattice *self, HklLattice *reciprocal)
 	s_beta2 = D / s_gamma_s_alpha;
 	s_beta3 = D / s_alpha_s_beta;
 
-	hkl_lattice_set(reciprocal,
-			HKL_TAU * s_alpha / (hkl_parameter_value_get(self->a, HKL_UNIT_DEFAULT) * D),
-			HKL_TAU * s_beta  / (hkl_parameter_value_get(self->b, HKL_UNIT_DEFAULT) * D),
-			HKL_TAU * s_gamma / (hkl_parameter_value_get(self->c, HKL_UNIT_DEFAULT) * D),
-			atan2(s_beta1, c_beta1),
-			atan2(s_beta2, c_beta2),
-			atan2(s_beta3, c_beta3),
-			HKL_UNIT_DEFAULT, NULL);
+	IGNORE(hkl_lattice_set(reciprocal,
+                               HKL_TAU * s_alpha / (hkl_parameter_value_get(self->a, HKL_UNIT_DEFAULT) * D),
+                               HKL_TAU * s_beta  / (hkl_parameter_value_get(self->b, HKL_UNIT_DEFAULT) * D),
+                               HKL_TAU * s_gamma / (hkl_parameter_value_get(self->c, HKL_UNIT_DEFAULT) * D),
+                               atan2(s_beta1, c_beta1),
+                               atan2(s_beta2, c_beta2),
+                               atan2(s_beta3, c_beta3),
+                               HKL_UNIT_DEFAULT, NULL));
 
 	return TRUE;
 }
@@ -665,8 +796,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute the alpha angle. */
-			hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		} else if (self->beta->fit) {
 			/* beta */
 			a = b = vector_x;
@@ -685,8 +816,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute beta */
-			hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		} else {
 			/* gamma */
 			a = c = vector_x;
@@ -705,8 +836,8 @@ void hkl_lattice_randomize(HklLattice *self)
 										 HKL_UNIT_DEFAULT));
 
 			/* compute gamma */
-			hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		}
 		break;
 	case 2:
@@ -724,10 +855,10 @@ void hkl_lattice_randomize(HklLattice *self)
 				/* randomize c */
 				hkl_vector_randomize_vector_vector(&c, &a, &b);
 
-				hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-							HKL_UNIT_DEFAULT, NULL);
-				hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-							HKL_UNIT_DEFAULT, NULL);
+				IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
+				IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
 			} else {
 				/* alpha + gamma */
 				a = c = vector_x;
@@ -741,10 +872,10 @@ void hkl_lattice_randomize(HklLattice *self)
 				/* randomize c */
 				hkl_vector_randomize_vector_vector(&b, &a, &c);
 
-				hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-							HKL_UNIT_DEFAULT, NULL);
-				hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-							HKL_UNIT_DEFAULT, NULL);
+				IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                                               HKL_UNIT_DEFAULT, NULL));
+				IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                               HKL_UNIT_DEFAULT, NULL));
 			}
 		} else {
 			/* beta + gamma */
@@ -759,10 +890,10 @@ void hkl_lattice_randomize(HklLattice *self)
 			/* randomize c */
 			hkl_vector_randomize_vector_vector(&a, &b, &c);
 
-			hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-						HKL_UNIT_DEFAULT, NULL);
-			hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-						HKL_UNIT_DEFAULT, NULL);
+			IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                                       HKL_UNIT_DEFAULT, NULL));
+			IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                                       HKL_UNIT_DEFAULT, NULL));
 		}
 		break;
 	case 3:
@@ -770,12 +901,12 @@ void hkl_lattice_randomize(HklLattice *self)
 		hkl_vector_randomize_vector(&b, &a);
 		hkl_vector_randomize_vector_vector(&c, &b, &a);
 
-		hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
-					HKL_UNIT_DEFAULT, NULL);
-		hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
-					HKL_UNIT_DEFAULT, NULL);
-		hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
-					HKL_UNIT_DEFAULT, NULL);
+		IGNORE(hkl_parameter_value_set(self->alpha, hkl_vector_angle(&b, &c),
+                                               HKL_UNIT_DEFAULT, NULL));
+		IGNORE(hkl_parameter_value_set(self->beta, hkl_vector_angle(&a, &c),
+                                               HKL_UNIT_DEFAULT, NULL));
+		IGNORE(hkl_parameter_value_set(self->gamma, hkl_vector_angle(&a, &b),
+                                               HKL_UNIT_DEFAULT, NULL));
 		break;
 	}
 }
