@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2025 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -126,24 +126,21 @@ static const HklFunction psi_constant_vertical_func = {
 	{								\
 		HKL_PARAMETER_DEFAULTS, .name = "h2", ._value = _h2,	\
 			.description = "h coordinate of the reference plan", \
-			.range = { .min=-1, .max=1 },			\
 			},						\
 	{								\
 		HKL_PARAMETER_DEFAULTS, .name = "k2", ._value = _k2,	\
 			.description = "k coordinate of the reference plan", \
-			.range = { .min=-1, .max=1 },			\
 			},						\
 	{								\
 		HKL_PARAMETER_DEFAULTS, .name = "l2", ._value = _l2,	\
 			.description = "l coordinate of the reference plan", \
-			.range = { .min=-1, .max=1 },			\
 			},						\
 	{								\
 		HKL_PARAMETER_DEFAULTS_ANGLE, .name = "psi", ._value = _psi, \
 			.description = "expected angle between the reference and the diffraction plans", \
 			}
 
-	static const HklParameter constant_omega_parameters[] = { CONSTANT_PARAMETER(omega) };
+static const HklParameter constant_omega_parameters[] = { CONSTANT_PARAMETER(omega) };
 static const HklParameter constant_chi_parameters[] = { CONSTANT_PARAMETER(chi) };
 static const HklParameter constant_phi_parameters[] = { CONSTANT_PARAMETER(phi) };
 
@@ -152,17 +149,14 @@ static const HklParameter double_diffraction_parameters[] = {
 	{
 		HKL_PARAMETER_DEFAULTS, .name = "h2", ._value = 1,
 		.description = "h coordinate of the second diffracting plan",
-		.range = {.min=-1, .max=1},
 	},
 	{
 		HKL_PARAMETER_DEFAULTS, .name = "k2", ._value = 1,
 		.description = "k coordinate of the second diffracting plan",
-		.range = {.min=-1, .max=1},
 	},
 	{
 		HKL_PARAMETER_DEFAULTS, .name = "l2", ._value = 1,
 		.description = "l coordinate of the second diffracting plan",
-		.range = {.min=-1, .max=1},
 	},
 };
 
