@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2022, 2025, 2026 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -132,7 +132,7 @@ static int fit_detector_position(HklMode *mode,
 	params.geometry = geometry;
 	params.detector = detector;
 	params.kf0 = kf;
-	params.axes = malloc(sizeof(*params.axes) * detector_holder->config->len);
+	params.axes = g_new (HklParameter *, detector_holder->config->len);
 	params.len = 0;
 	/* for each axis of the mode */
 	darray_foreach(axis_name, mode->info->axes_w){
@@ -410,7 +410,7 @@ int hkl_mode_set_hkl_real(HklMode *self,
 
 		/* we will add solution to the geometries so save its length before */
 		for(i=0, item=list_top(&engine->engines->geometries->items, HklGeometryListItem, list);
-		    i<len;
+		    i<len && NULL != item;
 		    ++i, item=list_next(&engine->engines->geometries->items, item, list)){
 			int j;
 			HklVector ki;
@@ -806,7 +806,7 @@ HklMode *hkl_mode_hkl_emergence_fixed_new(const HklModeAutoInfo *auto_info)
 		exit(128);
 	}
 
-	self = HKL_MALLOC(HklModeAutoHklEmergenceFixed);
+	self = g_new(HklModeAutoHklEmergenceFixed, 1);
 
 	/* the base constructor; */
 	hkl_mode_auto_init(&self->parent,
@@ -834,21 +834,18 @@ static void hkl_engine_hkl_free_real(HklEngine *base)
 
 HklEngine *hkl_engine_hkl_new(HklEngineList *engines)
 {
-	HklEngineHkl *self;
+	HklEngineHkl *self = g_new(HklEngineHkl, 1);
 	static const HklParameter h = {
 		HKL_PARAMETER_DEFAULTS, .name = "h",
 		.description = "h coordinate of the diffracting plan",
-		.range = { .min=-1, .max=1 },
 	};
 	static const HklParameter k = {
 		HKL_PARAMETER_DEFAULTS, .name = "k",
 		.description = "k coordinate of the diffracting plan",
-		.range = { .min=-1, .max=1 },
 	};
 	static const HklParameter l = {
 		HKL_PARAMETER_DEFAULTS, .name = "l",
 		.description = "l coordinate of the diffracting plan",
-		.range={ .min=-1, .max=1 },
 	};
 	static const HklParameter *pseudo_axes[] = {&h, &k, &l};
 	static HklEngineInfo info = {
@@ -861,8 +858,6 @@ HklEngine *hkl_engine_hkl_new(HklEngineList *engines)
 		.free=hkl_engine_hkl_free_real,
 	};
 
-	self = HKL_MALLOC(HklEngineHkl);
-
 	hkl_engine_init(&self->engine, &info, &operations, engines);
 
 	self->h = register_pseudo_axis(&self->engine, engines, &h);
