@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2021 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -34,7 +34,8 @@ static void degenerated(void)
 	HklDetector *detector;
 	HklSample *sample;
 	static double hkl[] = {0, 1, 0};
-	struct Geometry gconf = K4cv(1.54, 30., 0., 0., 60.);
+	Geometry gconf = K4cv(1.54, VALUES(30., 0., 0., 60.));
+        struct Sample cu = CU;
 
 	geometry = newGeometry(gconf);
 	engines = newEngines(gconf);
@@ -98,7 +99,8 @@ static void eulerians(void)
 	HklDetector *detector;
 	HklSample *sample;
 	static double eulerians[] = {0., 90 * HKL_DEGTORAD, 0.};
-	struct Geometry gconf = K4cv(1.54, 0., 0., 0., 0.);
+	Geometry gconf = K4cv(1.54, VALUES(0., 0., 0., 0.));
+        struct Sample cu = CU;
 
 	geometry = newGeometry(gconf);
 	engines = newEngines(gconf);
@@ -171,7 +173,8 @@ static void q(void)
 	HklGeometry *geometry;
 	HklDetector *detector;
 	HklSample *sample;
-	struct Geometry gconf = K4cv(1.54, 30., 0., 0., 60.);
+	Geometry gconf = K4cv(1.54, VALUES(30., 0., 0., 60.));
+        struct Sample cu = CU;
 
 	geometry = newGeometry(gconf);
 	engines = newEngines(gconf);
